import ipaddress
import json
from datetime import datetime
from typing import Dict

from pymeshviewer import Protocol
from pymeshviewer.graph import Node as GraphNode, LinkType
from pymeshviewer.graph import NodeGraph, ProtocolBatmanAdv, Link
from pymeshviewer.meshviewerjson import MeshviewerJSON
from pymeshviewer.node import Network, Location, Autoupdater, Firmware, Statistics, Nodeinfo, System, Software, \
    Hardware, Node, Traffic, Processes, Mesh, BatmanAdv, Fastd, StatusPage, MeshVPNPeer, MeshVPNPeerGroup, MeshVPN, \
    Neighbour
from pymeshviewer.nodesjson import NodesJSON


def parse_datetime(datetime_string: str) -> datetime:
    """
    Parses the datetime string from meshviewer json and returns it as a datetime object
    :param datetime_string: datetime string from yanic json
    :return: parsed datetime string as datetime object
    """
    return datetime.strptime(datetime_string, '%Y-%m-%dT%H:%M:%S%z')


def parse_network(network_dict: dict) -> Network:
    """
    Parses the provided network dict in ffrgb-meshviewer format
    :param network_dict: network section dict
    :return: a Network object containing the information from the input
    """
    mesh = None
    if "mesh" in network_dict:
        mesh = parse_mesh_interfaces(network_dict["mesh"])
    output = Network(mac=network_dict["mac"],
                     addresses=list(map(lambda x: ipaddress.ip_address(x), network_dict["addresses"])),
                     mesh=mesh)
    return output


def parse_location(location_dict: dict) -> Location:
    """
    Parses the provided location dict in ffrgb-meshviewer format
    :param location_dict: location section dict
    :return: a Location object containing the information from the input
    """
    output = Location()
    output.latitude = location_dict.get("latitude", 0)
    output.longitude = location_dict.get("longitude", 0)
    return output


def parse_autoupdater(autoupdater_dict: dict):
    return Autoupdater(enabled=autoupdater_dict.get("enabled", False), branch=autoupdater_dict.get("branch", None))


def parse_firmware(firmware_dict: Dict[str, str]):
    return Firmware(base=firmware_dict.get("base", None), release=firmware_dict.get("release", None))


def parse_graph_json(input_string: str) -> NodeGraph:
    """
    Parses the provided node graph json string and returns it as a NodeGraph object
    :param input_string: node graph json string
    :return: a NodeGraph object containing the information from the input
    """
    return parse_node_graph(json.loads(input_string))


def parse_node_graph(input_dict: dict) -> NodeGraph:
    """
    Parses the provided node graph dict and returns it as a NodeGraph object
    :param input_dict: node graph dict
    :return: a NodeGraph object containing the information from the input
    """
    return NodeGraph(version=input_dict["version"], protocol=parse_batadv_graph(input_dict["batadv"]),
                     protocol_type=Protocol.BATMAN_ADV)


def parse_graph_node(input_dict: dict) -> Node:
    """
    Parses the provided node dict and returns it as a Node object
    :param input_dict: node dict
    :return: a Node object containing the information from the input
    """
    return GraphNode(id=input_dict["id"], node_id=input_dict["node_id"])


def parse_batadv_graph(input_dict: dict) -> ProtocolBatmanAdv:
    """
    Parses the provided batman-adv protocol dict and returns it as a ProtocolBatmanAdv object
    :param input_dict: batman-adv protocol dict
    :return: a ProtocolBatmanAdv object containing the information from the input
    """
    nodes = list(map(lambda x: parse_graph_node(x), input_dict["nodes"]))
    return ProtocolBatmanAdv(directed=input_dict["directed"], graph=input_dict["graph"], nodes=nodes,
                             links=list(map(lambda x: parse_graph_link(x, nodes), input_dict["links"])))


def parse_graph_link(input_dict: dict, nodes: list) -> Link:
    """
    Parses the provided graph link dict and returns it as a ProtocolBatmanAdv object
    :param input_dict: batman-adv protocol dict
    :param nodes: list of all graph nodes correctly ordered
    :return: a Link object containing the information from the input
    """
    return Link(source=nodes[input_dict["source"]], target=nodes[input_dict["target"]], vpn=input_dict["vpn"],
                tq=input_dict["tq"], bidirect=input_dict["bidirect"])


def parse_nodesjson_node(node_dict: dict) -> Node:
    """
    Parses the provided node-dict in ffrgb-meshviewer format
    :param node_dict: dict containing a single object of node type
    :return: a Node object containing the information from the input
    """
    output = Node(
        firstseen=parse_datetime(node_dict['firstseen']),
        lastseen=parse_datetime(node_dict['lastseen']),
        online=node_dict["flags"]["online"],
        gateway=node_dict["flags"]["gateway"],
        statistics=parse_statistics(node_dict["statistics"]),
        nodeinfo=parse_nodeinfo(node_dict["nodeinfo"]))
    return output


def parse_nodes_json(nodes_str: str) -> NodesJSON:
    """
    Parses the provided nodelist json in ffrgb-meshviewer format
    :param nodes_str: nodelist json
    :return: a Nodelist object containing the information from the input
    """
    root_obj = json.loads(nodes_str)
    return NodesJSON(list(map(lambda x: parse_nodesjson_node(x), root_obj["nodes"])), root_obj["version"],
                     parse_datetime(
                         root_obj["timestamp"]))


def parse_traffic(traffic_dict: dict) -> Traffic:
    """
    Parses the provided traffic dict in ffrgb-meshviewer format
    :param traffic_dict: traffic section dict
    :return: a Traffic object containing the information from the input
    """
    return Traffic(bytes=traffic_dict.get("bytes", None),
                   packets=traffic_dict.get("packets", None),
                   dropped=traffic_dict.get("dropped", None))


def parse_processes(processes_dict: dict) -> Processes:
    """
    Parses the provided processes dict in ffrgb-meshviewer format
    :param processes_dict: processes section dict
    :return: a Processes object containing the information from the input
    """
    return Processes(total=processes_dict["total"], running=processes_dict["running"])


def parse_statistics(statistics_dict: dict) -> Statistics:
    """
    Parses the provided statistics dict in ffrgb-meshviewer format
    :param statistics_dict: statistics section dict
    :return: a Statistics object containing the information from the input
    """
    mesh_vpn = None
    if "mesh_vpn" in statistics_dict:
        mesh_vpn = parse_mesh_vpn(statistics_dict["mesh_vpn"])

    output = Statistics(node_id=statistics_dict["node_id"], clients=statistics_dict["clients"],
                        rootfs_usage=statistics_dict.get("rootfs_usage", None),
                        loadavg=statistics_dict.get("loadavg", None), memory_usage=statistics_dict["memory_usage"],
                        uptime=statistics_dict["uptime"], idletime=statistics_dict["idletime"],
                        gateway=statistics_dict.get("gateway", None),
                        processes=parse_processes(statistics_dict["processes"]),
                        mesh_vpn=mesh_vpn,
                        tx=parse_traffic(statistics_dict["traffic"]["tx"]),
                        rx=parse_traffic(statistics_dict["traffic"]["rx"]),
                        forward=parse_traffic(statistics_dict["traffic"]["forward"]),
                        mgmt_tx=parse_traffic(statistics_dict["traffic"]["mgmt_tx"]),
                        mgmt_rx=parse_traffic(statistics_dict["traffic"]["mgmt_rx"]))
    return output


def parse_mesh_interfaces(mesh_interfaces_dict: dict) -> dict:
    """
    Parses the provided mesh-interfaces dict in ffrgb-meshviewer format
    :param mesh_interfaces_dict: mesh-interfaces section dict
    :return: a dict containing the information from the input as Mesh type
    """
    output = {}
    for k, v in mesh_interfaces_dict.items():
        v = v["interfaces"]
        wireless = []
        tunnel = []
        other = []
        if 'wireless' in v:
            wireless = v["wireless"]
        if 'tunnel' in v:
            tunnel = v["tunnel"]
        if 'other' in v:
            other = v["other"]
        output[k] = Mesh(wireless=wireless, tunnel=tunnel, other=other)
    return output


def parse_system(system_dict: dict) -> System:
    """
    Parses the provided system dict in ffrgb-meshviewer format
    :param system_dict: system section dict
    :return: a System object containing the information from the input
    """
    return System(system_dict.get("site_code", None))


def parse_software(software_dict: dict) -> Software:
    """
    Parses the provided software dict in ffrgb-meshviewer format
    :param software_dict: software section dict
    :return: a Software object containing the information from the input
    """
    output = Software()
    for k, v in software_dict.items():
        if k == "autoupdater":
            output.autoupdater = parse_autoupdater(v)
        elif k == "batman-adv":
            output.batman_adv = BatmanAdv(version=v.get("version", None), compat=v.get("compat", None))
        elif k == "fastd":
            output.fastd = Fastd(enabled=v.get("enabled", False), version=v.get("version", None))
        elif k == "firmware":
            output.firmware = parse_firmware(v)
        elif k == "status-page":
            output.status_page = StatusPage(v.get("api", None))
    return output


def parse_hardware(hardware_dict: dict) -> Hardware:
    """
    Parses the provided hardware dict in ffrgb-meshviewer format
    :param hardware_dict: hardware section dict
    :return: a Hardware object containing the information from the input
    """
    return Hardware(nproc=hardware_dict.get("nproc", None), model=hardware_dict.get("model", None))


def parse_mesh_vpn_peer(peer_dict: dict) -> MeshVPNPeer:
    """
    Parses the provided peer section dict in ffrgb-meshviewer format
    :param peer_dict: peer section dict
    :return: a MeshVPNPeer object containing the information from the input
    """
    if peer_dict is None:
        return MeshVPNPeer(established=False)
    return MeshVPNPeer(established=True, established_time=peer_dict["established"])


def parse_mesh_vpn_group(group_dict: dict) -> MeshVPNPeerGroup:
    """
    Parses the provided mesh-vpn group section dict in ffrgb-meshviewer format
    :param group_dict: group section dict
    :return: a MeshVPNPeerGroup object containing the information from the input
    """
    peers = {}
    for k, v in group_dict["peers"].items():
        peers[k] = parse_mesh_vpn_peer(v)
    return MeshVPNPeerGroup(peers)


def parse_mesh_vpn(mesh_vpn_dict: dict) -> MeshVPN:
    """
    Parses the provided mesh-vpn section dict in ffrgb-meshviewer format
    :param mesh_vpn_dict: mesh-vpn section dict
    :return: a MeshVPN object containing the information from the input
    """
    groups = {}
    for k, v in mesh_vpn_dict["groups"].items():
        groups[k] = parse_mesh_vpn_group(v)
    return MeshVPN(groups)


def parse_nodeinfo(nodeinfo_dict: dict) -> Nodeinfo:
    """
    Parses the provided nodeinfo section dict in ffrgb-meshviewer format
    :param nodeinfo_dict: nodeinfo section dict
    :return: a Nodeinfo object containing the information from the input
    """
    output = Nodeinfo(
        node_id=nodeinfo_dict["node_id"],
        network=parse_network(nodeinfo_dict["network"]),
        hostname=nodeinfo_dict["hostname"],
        software=parse_software(nodeinfo_dict["software"]),
        hardware=parse_hardware(nodeinfo_dict["hardware"]),
        system=parse_system(nodeinfo_dict["system"]),
        vpn=nodeinfo_dict["vpn"])
    if "location" in nodeinfo_dict.keys():
        output.location = parse_location(nodeinfo_dict["location"])
    return output


def parse_meshviewer_json(input_str: str) -> MeshviewerJSON:
    root_obj = json.loads(input_str)
    nodes = list(map(lambda x: parse_meshviewer_node(x), root_obj["nodes"]))
    links = list(map(lambda x: parse_meshviewer_link(x), root_obj["links"]))

    meshviewer_json = MeshviewerJSON(nodes=nodes, links=links, timestamp=parse_datetime(root_obj["timestamp"]))
    for link in links:
        src = meshviewer_json.get_node(link.source)
        dst = meshviewer_json.get_node(link.target)

        src.neighbours.append(Neighbour(protocol=Protocol.GENERAL, node_id=link.target, vpn=link.vpn, tq=link.tq))
        dst.neighbours.append(Neighbour(protocol=Protocol.GENERAL, node_id=link.source, vpn=link.vpn, tq=link.tq))

    return meshviewer_json


def parse_meshviewer_link_type(input_str: str) -> LinkType:
    """
    Returns the Meshviewer.json link type as a LinkType
    :param input_str: link type from meshviewer
    :return: LinkType
    """
    if input_str == "wifi":
        return LinkType.WIFI
    if input_str == "vpn":
        return LinkType.VPN
    return LinkType.OTHER


def parse_meshviewer_link(link_dict: dict) -> Link:
    """
    Parses a Meshviewer.json link
    :param link_dict: link
    :return: Parsed link
    """
    return Link(source=link_dict["source"],
                target=link_dict["target"],
                source_tq=link_dict["source_tq"],
                target_tq=link_dict["target_tq"],
                link_type=parse_meshviewer_link_type(link_dict["type"]))


def parse_meshviewer_node(node_dict: dict) -> Node:
    """
    Parses a node from meshviewer.json
    :param node_dict: node
    :return: Parsed node of type node
    """
    autoupdater = None
    if "autoupdater" in node_dict:
        autoupdater = parse_autoupdater(node_dict["autoupdater"])

    firmware = None
    if "firmware" in node_dict:
        firmware = parse_firmware(node_dict["firmware"])

    statistics = Statistics(node_id=node_dict["node_id"],
                            clients=node_dict["clients"],
                            clients_wifi24=node_dict["clients_wifi24"],
                            clients_wifi5=node_dict["clients_wifi5"],
                            clients_other=node_dict["clients_other"],
                            rootfs_usage=node_dict.get("rootfs_usage", 0),
                            loadavg=node_dict.get("loadavg", None),
                            memory_usage=node_dict["memory_usage"],
                            uptime=int((parse_datetime(node_dict["lastseen"]) - parse_datetime(
                                node_dict["uptime"])).total_seconds()),
                            idletime=0,
                            gateway=node_dict.get("gateway", None)
                            )
    location = None
    if "location" in node_dict:
        location = parse_location(node_dict["location"])
    nodeinfo = Nodeinfo(node_id=node_dict["node_id"],
                        network=parse_network({"addresses": node_dict["addresses"], "mac": node_dict["mac"]}),
                        system=System(site_code=node_dict.get("site_code", None)),
                        hostname=node_dict["hostname"],
                        location=location,
                        software=Software(autoupdater=autoupdater,
                                          firmware=firmware),
                        hardware=Hardware(model=node_dict.get("model", None),
                                          nproc=node_dict["nproc"]),
                        vpn=node_dict["vpn"])
    return Node(firstseen=parse_datetime(node_dict["firstseen"], ),
                lastseen=parse_datetime(node_dict["lastseen"]),
                online=node_dict["is_online"],
                gateway=node_dict["is_gateway"],
                statistics=statistics,
                nodeinfo=nodeinfo
                )



import ipaddress
import unittest
from datetime import datetime

from pymeshviewer.node import Node, Nodeinfo, System, Hardware, Software, StatusPage, Firmware, MeshVPNPeer, \
    MeshVPNPeerGroup, MeshVPN, Traffic, Processes, Statistics, Mesh, Network, Location, Autoupdater, BatmanAdv, Fastd


class NodeTest(unittest.TestCase):
    def setUp(self):
        """
        Initialize Tests
        """

        # Peer
        self.peer_1_established = False
        self.peer_1 = MeshVPNPeer(established=self.peer_1_established)
        self.peer_2_established = True
        self.peer_2_established_time = 300.33
        self.peer_2 = MeshVPNPeer(established=self.peer_2_established, established_time=self.peer_2_established_time)

        # Peer Group
        self.peers_dict = {"peer1": self.peer_1, "peer2": self.peer_2}
        self.peer_group = MeshVPNPeerGroup(self.peers_dict)

        # Mesh VPN
        self.mesh_vpn_groups = {"backbone": self.peer_group}
        self.mesh_vpn = MeshVPN(self.mesh_vpn_groups)

        # Traffic
        self.tx_bytes = 500
        self.tx_dropped = 10
        self.tx_packets = 50
        self.tx = Traffic(bytes=self.tx_bytes, dropped=self.tx_dropped, packets=self.tx_packets)

        self.rx_bytes = 553
        self.rx_dropped = 99
        self.rx_packets = 399
        self.rx = Traffic(bytes=self.rx_bytes, dropped=self.rx_dropped, packets=self.rx_packets)

        self.forward_bytes = 53040
        self.forward_dropped = 944
        self.forward_packets = 3244
        self.forward = Traffic(bytes=self.forward_bytes, dropped=self.forward_dropped, packets=self.forward_packets)

        self.mgmt_tx_bytes = 324234
        self.mgmt_tx_dropped = 232
        self.mgmt_tx_packets = 2354
        self.mgmt_tx = Traffic(bytes=self.mgmt_tx_bytes, dropped=self.mgmt_tx_dropped, packets=self.mgmt_tx_packets)

        self.mgmt_rx_bytes = 5241
        self.mgmt_rx_dropped = 34
        self.mgmt_rx_packets = 647
        self.mgmt_rx = Traffic(bytes=self.mgmt_rx_bytes, dropped=self.mgmt_rx_dropped, packets=self.mgmt_rx_packets)

        # Processes
        self.processes_total = 500
        self.processes_running = 100
        self.processes = Processes(total=self.processes_total, running=self.processes_running)

        # Statistics
        self.node_id = "deadbeefffda"
        self.client_count = 5
        self.rootfs_usage = 0.4124
        self.load = 0.45243242
        self.memory_usage = 0.55
        self.uptime = 133842
        self.idletime = 400
        self.gateway = "de:ad:be:ef:ff:ff"
        self.statistics = Statistics(node_id=self.node_id, clients=self.client_count, rootfs_usage=self.rootfs_usage,
                                     loadavg=self.load, memory_usage=self.memory_usage, uptime=self.uptime,
                                     idletime=self.idletime, gateway=self.gateway, processes=self.processes,
                                     mesh_vpn=self.mesh_vpn, tx=self.tx, rx=self.rx,
                                     forward=self.forward, mgmt_tx=self.mgmt_tx, mgmt_rx=self.mgmt_rx)
        # Mesh
        self.mesh_ifs_wireless = ["de:ad:be:ef:f0:da", "de:ad:be:ef:f1:da"]
        self.mesh_ifs_tunnel = ["de:ad:be:ef:f1:df"]
        self.mesh_ifs_other = ["de:ad:be:ef:f1:de"]
        self.mesh_ifs = Mesh(wireless=self.mesh_ifs_wireless, tunnel=self.mesh_ifs_tunnel, other=self.mesh_ifs_other)

        # Network
        self.mac = "de:ad:be:ef:ff:da"
        self.v6_address_global = ipaddress.ip_address("2001:67C:2ED8:6100:DCAD:BEFF:FEEF:FFDA".lower())
        self.v6_address_ula = ipaddress.ip_address("fd01:67C:2ED8:6100:DCAD:BEFF:FEEF:FFDA".lower())
        self.v6_address_link_local = ipaddress.ip_address("fe80::DCAD:BEFF:FEEF:FFDA".lower())
        self.addresses = [
            self.v6_address_global,
            self.v6_address_ula,
            self.v6_address_link_local
        ]
        self.mesh = {"bat0": self.mesh_ifs}
        self.network = Network(mac=self.mac, addresses=self.addresses, mesh=self.mesh)

        # System
        self.site_code = "ffda"
        self.system = System(self.site_code)

        # Location
        self.latitude = 49.849
        self.longitude = 8.733
        self.altitude = 19.4
        self.location = Location(latitude=self.latitude, longitude=self.longitude, altitude=self.altitude)

        # Autoupdater
        self.autoupdater_enabled = True
        self.autoupdater_branch = "stable"
        self.autoupdater = Autoupdater(enabled=self.autoupdater_enabled, branch=self.autoupdater_branch)

        # Batman-Adv
        self.batman_adv_version = "2017.2"
        self.batman_adv_compat = 15
        self.batman_adv = BatmanAdv(version=self.batman_adv_version, compat=self.batman_adv_compat)

        # Fastd
        self.fastd_version = "v18"
        self.fastd = Fastd(version=self.fastd_version)

        # Firmware
        self.firmware_base = "gluon-v2017.1.3"
        self.firmware_release = "1.0.3"
        self.firmware = Firmware(base=self.firmware_base, release=self.firmware_release)

        # StatusPage
        self.status_page_api = 1
        self.status_page = StatusPage(api=self.status_page_api)

        # Software
        self.software = Software(autoupdater=self.autoupdater, batman_adv=self.batman_adv, fastd=self.fastd,
                                 firmware=self.firmware, status_page=self.status_page)
        # Hardware
        self.hardware_nproc = 1
        self.hardware_model = "TP-Link TL-WR1043N/ND v4"
        self.hardware = Hardware(nproc=self.hardware_nproc, model=self.hardware_model)

        # Nodeinfo
        self.hostname = "64283-test"
        self.vpn = False
        self.nodeinfo = Nodeinfo(node_id=self.node_id, network=self.network, system=self.system, hostname=self.hostname,
                                 location=self.location, software=self.software, hardware=self.hardware, vpn=self.vpn)

        # Node
        self.now = datetime.now()
        self.online = True
        self.gateway_bool = False
        self.node = Node(firstseen=self.now, lastseen=self.now, online=self.online, gateway=self.gateway_bool,
                         statistics=self.statistics,
                         nodeinfo=self.nodeinfo)

    def test_traffic(self):
        self.assertEqual(self.tx.bytes, self.tx_bytes)
        self.assertEqual(self.tx.packets, self.tx_packets)
        self.assertEqual(self.tx.dropped, self.tx_dropped)

        self.assertEqual(self.rx.bytes, self.rx_bytes)
        self.assertEqual(self.rx.packets, self.rx_packets)
        self.assertEqual(self.rx.dropped, self.rx_dropped)

        self.assertEqual(self.forward.bytes, self.forward_bytes)
        self.assertEqual(self.forward.packets, self.forward_packets)
        self.assertEqual(self.forward.dropped, self.forward_dropped)

        self.assertEqual(self.mgmt_tx.bytes, self.mgmt_tx_bytes)
        self.assertEqual(self.mgmt_tx.packets, self.mgmt_tx_packets)
        self.assertEqual(self.mgmt_tx.dropped, self.mgmt_tx_dropped)

        self.assertEqual(self.mgmt_rx.bytes, self.mgmt_rx_bytes)
        self.assertEqual(self.mgmt_rx.packets, self.mgmt_rx_packets)
        self.assertEqual(self.mgmt_rx.dropped, self.mgmt_rx_dropped)

    def test_processes(self):
        self.assertEqual(self.processes.total, self.processes_total)
        self.assertEqual(self.processes.running, self.processes_running)

    def test_statistics(self):
        self.assertEqual(self.statistics.node_id, self.node_id)
        self.assertEqual(self.statistics.clients, self.client_count)
        self.assertEqual(self.statistics.rootfs_usage, self.rootfs_usage)
        self.assertEqual(self.statistics.loadavg, self.load)
        self.assertEqual(self.statistics.memory_usage, self.memory_usage)
        self.assertEqual(self.statistics.uptime, self.uptime)
        self.assertEqual(self.statistics.idletime, self.idletime)
        self.assertEqual(self.statistics.gateway, self.gateway)
        self.assertEqual(self.statistics.processes, self.processes)

        self.assertEqual(self.statistics.mesh_vpn, self.mesh_vpn)

        self.assertEqual(self.statistics.tx, self.tx)
        self.assertEqual(self.statistics.rx, self.rx)
        self.assertEqual(self.statistics.forward, self.forward)
        self.assertEqual(self.statistics.mgmt_rx, self.mgmt_rx)
        self.assertEqual(self.statistics.mgmt_tx, self.mgmt_tx)

    def test_mesh_ifs(self):
        self.assertEqual(self.mesh_ifs.tunnel, self.mesh_ifs_tunnel)
        self.assertEqual(self.mesh_ifs.wireless, self.mesh_ifs_wireless)
        self.assertEqual(self.mesh_ifs.other, self.mesh_ifs_other)

    def test_network(self):
        self.assertEqual(self.network.mac, self.mac)
        self.assertEqual(self.network.addresses, self.addresses)
        self.assertEqual(self.network.mesh, self.mesh)
        self.assertEqual(self.network.v6_addresses, self.addresses)
        self.assertEqual(self.network.v6_global, [self.v6_address_global])
        self.assertEqual(self.network.v6_ula, [self.v6_address_ula])
        self.assertEqual(self.network.v6_link_local, [self.v6_address_link_local])

    def test_system(self):
        self.assertEqual(self.system.site_code, self.site_code)

    def test_location(self):
        self.assertEqual(self.location.latitude, self.latitude)
        self.assertEqual(self.location.longitude, self.longitude)
        self.assertEqual(self.location.altitude, self.altitude)

    def test_software_autoupdater(self):
        self.assertEqual(self.autoupdater.enabled, self.autoupdater_enabled)
        self.assertEqual(self.autoupdater.branch, self.autoupdater_branch)

    def test_software_batman_adv(self):
        self.assertEqual(self.batman_adv.version, self.batman_adv_version)
        self.assertEqual(self.batman_adv.compat, self.batman_adv_compat)

    def test_software_fastd(self):
        self.assertEqual(self.fastd.version, self.fastd_version)

    def test_software_firmware(self):
        self.assertEqual(self.firmware.base, self.firmware_base)
        self.assertEqual(self.firmware.release, self.firmware_release)

    def test_software_status_page(self):
        self.assertEqual(self.status_page.api, self.status_page_api)

    def test_software(self):
        self.assertEqual(self.software.autoupdater, self.autoupdater)
        self.assertEqual(self.software.batman_adv, self.batman_adv)
        self.assertEqual(self.software.fastd, self.fastd)
        self.assertEqual(self.software.firmware, self.firmware)
        self.assertEqual(self.software.status_page, self.status_page)

    def test_hardware(self):
        self.assertEqual(self.hardware.nproc, self.hardware_nproc)
        self.assertEqual(self.hardware.model, self.hardware_model)

    def test_mesh_vpn_peer(self):
        self.assertEqual(self.peer_1.established, self.peer_1_established)
        self.assertEqual(self.peer_1.established_time, None)
        self.assertEqual(self.peer_2.established, self.peer_2_established)
        self.assertEqual(self.peer_2.established_time, self.peer_2_established_time)

    def test_mesh_vpn_group(self):
        self.assertEqual(self.peer_group.peers, self.peers_dict)

    def test_mesh_vpn(self):
        self.assertEqual(self.mesh_vpn.groups, self.mesh_vpn_groups)

    def test_nodeinfo(self):
        self.assertEqual(self.nodeinfo.node_id, self.node_id)
        self.assertEqual(self.nodeinfo.network, self.network)
        self.assertEqual(self.nodeinfo.system, self.system)
        self.assertEqual(self.nodeinfo.hostname, self.hostname)
        self.assertEqual(self.nodeinfo.location, self.location)
        self.assertEqual(self.nodeinfo.software, self.software)
        self.assertEqual(self.nodeinfo.hardware, self.hardware)
        self.assertEqual(self.nodeinfo.vpn, self.vpn)

    def test_node(self):
        self.assertEqual(self.node.firstseen, self.now)
        self.assertEqual(self.node.lastseen, self.now)
        self.assertEqual(self.node.online, self.online)
        self.assertEqual(self.node.gateway, self.gateway_bool)
        self.assertEqual(self.node.statistics, self.statistics)
        self.assertEqual(self.node.nodeinfo, self.nodeinfo)



from datetime import datetime
from typing import List

from pymeshviewer import Protocol
from pymeshviewer.graph import NodeGraph
from pymeshviewer.node import Neighbour, Node
from pymeshviewer.nodecollection import NodeCollection


class NodesJSON(NodeCollection):
    def __init__(self, nodes: List[Node], version: int, timestamp: datetime):
        """
        Constructor for a Nodelist
        :param nodes: list of nodes
        :param version: format revision
        :param timestamp: timestamp of nodelist
        """
        super().__init__(nodes)
        self.nodes = nodes
        self.version = version
        self.timestamp = None
        if timestamp is not None:
            self.timestamp = datetime.now()


    def load_nodegraph(self, graph: NodeGraph):
        """
        Loads nodegraph into nodelist and adds neighbours to nodes
        :param graph: corresponding nodegraph for nodelist
        """
        links = graph.protocol.links
        for link in links:
            source = self.get_node(link.source.node_id)
            target = self.get_node(link.target.node_id)
            if source and target:
                source.neighbours.append(
                    Neighbour(Protocol.BATMAN_ADV, target.nodeinfo.node_id, link.vpn,
                              int((float(2) - float(link.tq)) * 100),
                              link.bidirect))
                target.neighbours.append(
                    Neighbour(Protocol.BATMAN_ADV, source.nodeinfo.node_id, link.vpn,
                              int((float(2) - float(link.tq)) * 100),
                              link.bidirect))



import os
import unittest

from pymeshviewer.graph import LinkType
from pymeshviewer.parser import parse_meshviewer_json, parse_datetime


class MeshviewerJsonTest(unittest.TestCase):
    def setUp(self):
        nodelist_file = open(
            os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_meshviewer.json", "r")
        self.meshviewer_json = parse_meshviewer_json(nodelist_file.read())
        nodelist_file.close()

    def test_meshviewer_json(self):
        """
        Assert correct number of elements
        """
        self.assertEqual(len(self.meshviewer_json.nodes), 4)
        self.assertEqual(len(self.meshviewer_json.links), 3)

    def test_links(self):
        """
        Assert links are correctly parsed
        """
        self.assertEqual(self.meshviewer_json.links[0].link_type, LinkType.WIFI)
        self.assertEqual(self.meshviewer_json.links[0].source, "c04a00dd692a")
        self.assertEqual(self.meshviewer_json.links[0].target, "60e3272f92b2")
        self.assertEqual(self.meshviewer_json.links[0].source_tq, 0.8627451)
        self.assertEqual(self.meshviewer_json.links[0].target_tq, 0.9529412)

        self.assertEqual(self.meshviewer_json.links[1].link_type, LinkType.OTHER)
        self.assertEqual(self.meshviewer_json.links[1].source, "daff61000302")
        self.assertEqual(self.meshviewer_json.links[1].target, "daff61000402")
        self.assertEqual(self.meshviewer_json.links[1].source_tq, 1)
        self.assertEqual(self.meshviewer_json.links[1].target_tq, 1)

        self.assertEqual(self.meshviewer_json.links[2].link_type, LinkType.VPN)
        self.assertEqual(self.meshviewer_json.links[2].source, "daff61000302")
        self.assertEqual(self.meshviewer_json.links[2].target, "c04a00dd692a")
        self.assertEqual(self.meshviewer_json.links[2].source_tq, 1)
        self.assertEqual(self.meshviewer_json.links[2].target_tq, 1)

    def test_nodes(self):
        """
        Assert nodes are parsed correctly
        """
        node_0 = self.meshviewer_json.get_node("daff61000302")
        self.assertEqual(node_0.firstseen, parse_datetime("2016-11-27T03:52:55+0000"))
        self.assertEqual(node_0.lastseen, parse_datetime("2017-10-21T00:33:50+0200"))
        self.assertTrue(node_0.online)
        self.assertTrue(node_0.gateway)

        self.assertEqual(node_0.statistics.clients, 0)
        self.assertEqual(node_0.statistics.loadavg, 1.11)
        self.assertEqual(node_0.statistics.memory_usage, 0.34640929437841284)
        self.assertEqual(node_0.statistics.uptime, 20)
        self.assertEqual(node_0.statistics.node_id, "daff61000302")
        self.assertEqual(node_0.statistics.gateway, None)
        self.assertEqual(node_0.statistics.processes, None)
        self.assertEqual(node_0.statistics.mesh_vpn, None)
        self.assertEqual(node_0.statistics.rx, None)
        self.assertEqual(node_0.statistics.tx, None)
        self.assertEqual(node_0.statistics.forward, None)
        self.assertEqual(node_0.statistics.mgmt_rx, None)
        self.assertEqual(node_0.statistics.mgmt_tx, None)
        self.assertEqual(node_0.statistics.rootfs_usage, 0)

        self.assertEqual(node_0.nodeinfo.node_id, "daff61000302")
        self.assertEqual(node_0.nodeinfo.location, None)
        self.assertEqual(node_0.nodeinfo.hardware.model, None)
        self.assertEqual(node_0.nodeinfo.hardware.nproc, 3)
        self.assertEqual(node_0.nodeinfo.system.site_code, "ffda")
        self.assertEqual(node_0.nodeinfo.hostname, "gw03.darmstadt.freifunk.net")

        self.assertEqual(node_0.nodeinfo.software.autoupdater.enabled, False)
        self.assertEqual(node_0.nodeinfo.software.autoupdater.branch, None)

        self.assertEqual(node_0.nodeinfo.software.firmware.base, "Debian")
        self.assertEqual(node_0.nodeinfo.software.firmware.release, None)

        node_2 = self.meshviewer_json.get_node("c04a00dd692a")
        self.assertEqual(node_2.firstseen, parse_datetime("2015-10-16T11:00:02+0000"))
        self.assertEqual(node_2.lastseen, parse_datetime("2017-11-02T21:08:59+0100"))
        self.assertTrue(node_2.online)
        self.assertFalse(node_2.gateway)

        self.assertEqual(node_2.statistics.clients, 5)
        self.assertEqual(node_2.statistics.clients_wifi24, 1)
        self.assertEqual(node_2.statistics.clients_wifi5, 3)
        self.assertEqual(node_2.statistics.clients_other, 1)
        self.assertEqual(node_2.statistics.clients, 5)
        self.assertEqual(node_2.statistics.loadavg, 0.45)
        self.assertEqual(node_2.statistics.memory_usage, 0.2151159077846606)
        self.assertEqual(node_2.statistics.rootfs_usage, 0.0904)
        self.assertEqual(node_2.statistics.uptime, 93681)
        self.assertEqual(node_2.statistics.node_id, "c04a00dd692a")
        self.assertEqual(node_2.statistics.gateway, "daff61000605")
        self.assertEqual(node_2.statistics.processes, None)
        self.assertEqual(node_2.statistics.mesh_vpn, None)
        self.assertEqual(node_2.statistics.rx, None)
        self.assertEqual(node_2.statistics.tx, None)
        self.assertEqual(node_2.statistics.forward, None)
        self.assertEqual(node_2.statistics.mgmt_rx, None)
        self.assertEqual(node_2.statistics.mgmt_tx, None)

        self.assertEqual(node_2.nodeinfo.node_id, "c04a00dd692a")
        self.assertEqual(node_2.nodeinfo.location.latitude, 49.83675075)
        self.assertEqual(node_2.nodeinfo.location.longitude, 8.70046645)
        self.assertEqual(node_2.nodeinfo.hardware.model, "TP-Link TL-WDR3600 v1")
        self.assertEqual(node_2.nodeinfo.hardware.nproc, 1)
        self.assertEqual(node_2.nodeinfo.system.site_code, "ffda")
        self.assertEqual(node_2.nodeinfo.hostname, "64367-blockheizkraftwerk")

        self.assertEqual(node_2.nodeinfo.software.autoupdater.enabled, True)
        self.assertEqual(node_2.nodeinfo.software.autoupdater.branch, "stable")

        self.assertEqual(node_2.nodeinfo.software.firmware.base, "gluon-v2017.1.3")
        self.assertEqual(node_2.nodeinfo.software.firmware.release, "1.0.3")



import ipaddress
from datetime import datetime
from typing import List, Dict

from pymeshviewer import Protocol


class MeshVPNPeer:
    def __init__(self, established: bool, established_time: float = None):
        """
        Constructor for a MeshVPNPeer object
        :param established: true if connection to peer is established, false if not
        :param established_time: if connection is established, connection time as a float
        """
        self.established = established
        self.established_time = established_time


class MeshVPNPeerGroup:
    def __init__(self, peers: Dict[str, MeshVPNPeer] = None):
        """
        Constructor for a MeshVPNPeerGroup object
        :param peers: dict of peers
        """
        if peers is None:
            peers = {}
        self.peers = peers


class MeshVPN:
    def __init__(self, groups: Dict[str, MeshVPNPeerGroup] = None):
        """
        Constructor for a MeshVPN object
        :param groups: dict of groups
        """
        if groups is None:
            groups = {}
        self.groups = groups


class Traffic:
    def __init__(self, bytes: int, packets: int, dropped: int):
        """
        Constructor for a Traffic object
        :param bytes: number of transferred bytes
        :param packets: number of transferred packets
        :param dropped: number of dropped packets
        """
        self.bytes = bytes
        self.packets = packets
        self.dropped = dropped


class Processes:
    def __init__(self, total: int, running: int):
        """
        Constructor for a Processes object
        :param total: number of total processes
        :param running: number of running processes
        """
        self.total = total
        self.running = running


class Statistics:
    def __init__(self, node_id: str, clients: int = None, clients_wifi24: int = None, clients_wifi5: int = None,
                 clients_other: int = None, rootfs_usage: float = None, loadavg: float = None,
                 memory_usage: float = None, uptime: int = None, idletime: int = None, gateway: str = None,
                 processes: Processes = None, mesh_vpn: MeshVPN = None,
                 tx: Traffic = None, rx: Traffic = None, forward: Traffic = None, mgmt_tx: Traffic = None,
                 mgmt_rx: Traffic = None):
        """
        Constructor for a Statistics object
        :param node_id: node_id of node
        :param clients: total number of clients
        :type clients_wifi24: 2.4 GHz WiFi clients
        :type clients_wifi5: 5 GHz WiFi clients
        :type clients_other: Other clients
        :param rootfs_usage: rootfs usage (0.0 - 1.0)
        :param loadavg: average load
        :param memory_usage: memory usage (0.0 - 1.0)
        :param uptime: uptime of node
        :param idletime: node idle time
        :param gateway: selected gateway
        :param processes: processes on node
        :param mesh_vpn: mesh vpn peers
        :param tx: traffic tx
        :param rx: traffic rx
        :param forward: traffic fowarded
        :param mgmt_tx: traffic mgmt_tx
        :param mgmt_rx: traffic mgmt_rx
        """
        self.node_id = node_id
        self.clients = clients
        self.clients_wifi24 = clients_wifi24
        self.clients_wifi5 = clients_wifi5
        self.clients_other = clients_other
        self.rootfs_usage = rootfs_usage
        self.loadavg = loadavg
        self.memory_usage = memory_usage
        self.uptime = uptime
        self.idletime = idletime
        self.gateway = gateway
        self.processes = processes
        self.mesh_vpn = mesh_vpn
        self.tx = tx
        self.rx = rx
        self.forward = forward
        self.mgmt_tx = mgmt_tx
        self.mgmt_rx = mgmt_rx


class System:
    def __init__(self, site_code: str):
        """
        Constructor for a System object
        :param site_code: system's site code
        """
        self.site_code = site_code


class Hardware:
    def __init__(self, nproc=0, model=None):
        """
        Constructor for Hardware object
        :param nproc: node nproc
        :param model: node model
        """
        self.nproc = nproc
        self.model = model


class Autoupdater:
    def __init__(self, enabled: bool, branch: str):
        """
        Constructor for Autoupdater object
        :param enabled: autoupdater status
        :param branch: selected branch
        """
        self.enabled = enabled
        self.branch = branch


class BatmanAdv:
    def __init__(self, version: str, compat: int):
        """
        Constructor for BatmanAdv object
        :param version: version of Batman-Adv
        :param compat: compat level
        """
        self.version = version
        self.compat = compat


class Fastd:
    def __init__(self, enabled: bool = False, version: str = None):
        """
        Constructor for Fastd object
        :param enabled: fastd status
        :param version: fastd version
        """
        self.enabled = enabled
        self.version = version


class Firmware:
    def __init__(self, base: str = None, release: str = None):
        """
        Constructor for Firmware object
        :param base: base gluon version
        :param release: custom version scheme
        """
        self.base = base
        self.release = release


class StatusPage:
    def __init__(self, api: int):
        """
        Constructor for StatusPage object
        :param api: api level
        """
        self.api = api


class Software:
    def __init__(self, autoupdater: Autoupdater = None, batman_adv: BatmanAdv = None, fastd: Fastd = None,
                 firmware: Firmware = None, status_page: StatusPage = None):
        """
        Constructor for Software object
        :param autoupdater: autoupdater software information
        :param batman_adv: batman-adv software information
        :param fastd: fastd software information
        :param firmware: firmware software information
        :param status_page: status page software information
        """
        self.autoupdater = autoupdater
        self.batman_adv = batman_adv
        self.fastd = fastd
        self.firmware = firmware
        self.status_page = status_page


class Mesh:
    def __init__(self, wireless: List[str] = None, tunnel: List[str] = None, other: List[str] = None):
        """
        Constructor for Mesh object
        :param wireless: list of wireless mesh interfaces
        :param tunnel: list of tunnel mesh interfaces
        :param other: list of other mesh interfaces
        """
        if other is None:
            other = []
        if tunnel is None:
            tunnel = []
        if wireless is None:
            wireless = []
        self.wireless = wireless
        self.tunnel = tunnel
        self.other = other


class Network:
    def __init__(self, mac: str, addresses: List[ipaddress._IPAddressBase], mesh: Dict[str, Mesh]):
        """
        Constructor for Network object
        :param mac: primary mac address of node
        :param addresses: list of nodes ip addresses
        :param mesh: mesh interfaces of node
        """
        if mesh is None:
            mesh = {}
        if addresses is None:
            addresses = []
        self.mac = mac
        self.addresses = addresses
        self.mesh = mesh

    @property
    def v6_addresses(self) -> List[ipaddress.IPv6Address]:
        """
        Nodes IPv6 addresses
        :return: ipv6 addresses of node
        """
        out = []
        for address in self.addresses:
            if address.version == 6:
                out.append(address)
        return out

    @property
    def v6_global(self) -> List[ipaddress.IPv6Address]:
        """
        Nodes globally routed IPv6 addresses
        :return: globally routed IPv6 addresses of node
        """
        out = []
        for address in self.v6_addresses:
            if address.is_global:
                out.append(address)
        return out

    @property
    def v6_ula(self) -> List[ipaddress.IPv6Address]:
        """
        Nodes ula IPv6 addresses
        :return: ULA IPv6 addresses of node
        """
        out = []
        for address in self.v6_addresses:
            if address.is_private and address not in self.v6_link_local:
                out.append(address)
        return out

    @property
    def v6_link_local(self) -> List[ipaddress.IPv6Address]:
        """
        Nodes Link-Local IPv6 addresses
        :return: Link-Local IPv6 addresses of node
        """
        out = []
        for address in self.v6_addresses:
            if address.is_link_local:
                out.append(address)
        return out


class Location:
    def __init__(self, latitude: float = 0, longitude: float = 0, altitude: float = 0):
        """
        Constructor for Location object
        :param latitude: nodes latitude
        :param longitude: nodes longitude
        :param altitude: nodes altitude
        """
        self.latitude = latitude
        self.longitude = longitude
        self.altitude = altitude


class Nodeinfo:
    def __init__(self, node_id: str, network: Network, system: System, hostname: str, vpn: bool,
                 software: Software = None, hardware: Hardware = None, location: Location = None):
        """
        Constructor for Nodeinfo object
        :param node_id: nodes node_id
        :param network: Network object of node
        :param system: System object of node
        :param hostname: nodes hostname
        :param software: Software object of node
        :param hardware: hardware object of node
        :param vpn: boolean indicating vpn server
        :param location: Location object of node
        """
        self.node_id = node_id
        self.network = network
        self.system = system
        self.hostname = hostname
        self.location = location
        self.software = software
        self.hardware = hardware
        self.vpn = vpn


class Neighbour:
    def __init__(self, protocol: Protocol, node_id: str, vpn: bool = False, tq: int = 0, bidirect: bool = False):
        """
        Constructor of Neighbour object
        :param protocol: mesh protocol
        :param node_id: neighbours node_id
        :param vpn: indicating connection is established via vpn
        :param tq: transmission quality (1.0-2.0)
        :param bidirect: bool indicating a bidirectional link
        """
        self.protocol = protocol
        self.node_id = node_id
        self.vpn = vpn
        self.tq = tq
        self.bidirect = bidirect


class Node:
    def __init__(self, firstseen: datetime, lastseen: datetime, online: bool, gateway: bool, statistics: Statistics,
                 nodeinfo: Nodeinfo, neighbours: List[Neighbour] = None):
        """
        Constructor for Node object
        :param firstseen: datetime of nodes first sight
        :param lastseen: datetime of nodes last sight
        :param online: bool indicating online status of node
        :param gateway: bool indicating if node is a gateway
        :param statistics: Statistics object of node
        :param nodeinfo: Nodeinfo object of node
        :param neighbours: nodes neighbours
        """
        if neighbours is None:
            neighbours = []
        self.firstseen = firstseen
        self.lastseen = lastseen
        self.online = online
        self.gateway = gateway
        self.statistics = statistics
        self.nodeinfo = nodeinfo
        self.neighbours = neighbours

    @property
    def vpn_enabled(self):
        """
        Return VPN enabled status
        :return: vpn enabled status
        """
        try:
            if self.nodeinfo.software.fastd.enabled is True:
                return True
        except AttributeError as e:
            pass
        return False

    @property
    def vpn_active(self):
        """
        Return VPN established status
        :return: vpn established status
        """
        if self.nodeinfo.vpn:
            return False
        if self.statistics.mesh_vpn is not None:
            for k, v in self.statistics.mesh_vpn.groups.items():
                for k, v in v.peers.items():
                    if v and v.established is True:
                        return True
        for n in self.neighbours:
            if n.vpn:
                return True
        return False



import os
import unittest

from pymeshviewer.parser import parse_nodes_json, parse_graph_json


class NodelistTest(unittest.TestCase):
    def setUp(self):
        """
        Initialize Tests
        """
        nodelist_file = open(
            os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_nodes.json", "r")
        self.nodelist = parse_nodes_json(nodelist_file.read())
        nodelist_file.close()

        nodegraph_file = open(
            os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_graph.json", "r")
        self.nodegraph = parse_graph_json(nodegraph_file.read())
        nodegraph_file.close()

    def test_get_node(self):
        """
        Verify that get_node() returns the correct node
        """
        self.assertEqual(self.nodelist.get_node("daff61000302"), self.nodelist.nodes[0])
        self.assertEqual(self.nodelist.get_node("daff61000402"), self.nodelist.nodes[1])
        self.assertEqual(self.nodelist.get_node("c04a00dd692a"), self.nodelist.nodes[2])
        self.assertEqual(self.nodelist.get_node("60e3272f92b2"), self.nodelist.nodes[3])
        self.assertIsNone(self.nodelist.get_node("000000000000"))

    def test_enabled_vpn_nodes(self):
        """
        Verify correct return value for vpn enabled nodes
        """
        self.assertEqual(len(self.nodelist.vpn_enabled), 3)

    def test_established_vpn_nodes(self):
        """
        Verify correct return value for vpn established nodes
        """
        self.assertEqual(len(self.nodelist.established_vpn_connection), 1)

    def test_model_stats(self):
        """
        Verify correct return value for model stats
        """
        self.assertDictEqual(self.nodelist.model_stats, {'TP-Link TL-WDR3600 v1': 1, 'TP-Link Archer C7 v2': 1})

    def test_load_nodegraph(self):
        """
        Test loading neighbours from Node Graph into nodelist
        """
        self.nodelist.load_nodegraph(self.nodegraph)

        self.assertEqual(len(self.nodelist.get_node("daff61000302").neighbours), 2)



import os
import unittest

from pymeshviewer.parser import parse_graph_json


class NodegraphParserTest(unittest.TestCase):
    def setUp(self):
        """
        Initialize Tests
        """

        file = open(
            os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_graph.json", "r")
        self.file_content = file.read()
        self.community = parse_graph_json(self.file_content)
        file.close()

    def test_version(self):
        """
        Test nodelist graph json version
        """
        self.assertEqual(self.community.version, 1)

    def test_batadv_directed(self):
        """
        Test nodelist graph batadv directed
        """
        self.assertEqual(self.community.protocol.directed, False)

    def test_batadv_graph(self):
        """
        Test nodelist graph batadv graph
        """
        self.assertEqual(self.community.protocol.graph, None)

    def test_batadv_nodes_size(self):
        """
        Test nodelist graph batadv nodelist count
        """
        self.assertEqual(len(self.community.protocol.nodes), 4)

    def test_batadv_links_size(self):
        """
        Test nodelist graph batadv link count
        """
        self.assertEqual(len(self.community.protocol.links), 3)

    def test_batadv_nodes(self):
        """
        Test batadv nodelist graph nodes value
        """
        self.assertEqual(self.community.protocol.nodes[0].node_id, "daff61000302")
        self.assertEqual(self.community.protocol.nodes[0].id, "da:ff:61:00:03:02")
        self.assertEqual(self.community.protocol.nodes[1].node_id, "daff61000402")
        self.assertEqual(self.community.protocol.nodes[1].id, "da:ff:61:00:04:02")

    def test_batadv_links(self):
        """
        Test batadv nodelist graph links value
        """
        self.assertEqual(self.community.protocol.links[0].source.node_id, "daff61000302")
        self.assertEqual(self.community.protocol.links[0].target.node_id, "daff61000402")
        self.assertEqual(self.community.protocol.links[0].vpn, False)
        self.assertEqual(self.community.protocol.links[0].tq, 1)
        self.assertEqual(self.community.protocol.links[0].bidirect, True)



import math


def calculate_distance(latitude1, longitude1, latitude2, longitude2):
    """
    Calculates the distance between two coordinates in kilometers
    :param latitude1: Latitude of first point
    :param longitude1: Longitude of first point
    :param latitude2: Latitude of second point
    :param longitude2: Longitude of second point
    :return: Distance between points in kilometers
    """
    r = 6373.0

    lat1 = math.radians(latitude1)
    lon1 = math.radians(longitude1)
    lat2 = math.radians(latitude2)
    lon2 = math.radians(longitude2)

    dlon = lon2 - lon1
    dlat = lat2 - lat1

    a = math.sin(dlat / 2) ** 2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon / 2) ** 2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))

    return r * c



import ipaddress
import os
import unittest

from pymeshviewer.parser import parse_datetime, parse_nodes_json


class NodelistParserTest(unittest.TestCase):
    def setUp(self):
        """
        Initialize Tests
        """

        file = open(os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_nodes.json",
                    "r")
        self.file_content = file.read()
        self.nodelist = parse_nodes_json(self.file_content)
        file.close()

    def test_size(self):
        """
        Verify the number of parsed nodes is correct
        """
        self.assertEqual(self.nodelist.node_count, 4)

    def test_firstseen(self):
        self.assertEqual(self.nodelist.nodes[0].firstseen, parse_datetime("2016-11-27T03:52:55+0000"))

    def test_lastseen(self):
        self.assertEqual(self.nodelist.nodes[0].lastseen, parse_datetime("2017-10-21T00:33:50+0200"))

    def test_flags(self):
        self.assertTrue(self.nodelist.nodes[0].online)
        self.assertTrue(self.nodelist.nodes[0].gateway)

    def test_statistics(self):
        self.assertEqual(self.nodelist.nodes[0].statistics.node_id, "daff61000302")
        self.assertEqual(self.nodelist.nodes[0].statistics.clients, 0)
        self.assertEqual(self.nodelist.nodes[0].statistics.loadavg, 1.11)
        self.assertEqual(self.nodelist.nodes[0].statistics.memory_usage, 0.34640929437841284)
        self.assertEqual(self.nodelist.nodes[0].statistics.uptime, 1463731.06)
        self.assertEqual(self.nodelist.nodes[0].statistics.idletime, 2674140.98)
        self.assertEqual(self.nodelist.nodes[0].statistics.processes.total, 124)
        self.assertEqual(self.nodelist.nodes[0].statistics.processes.running, 4)

        self.assertEqual(self.nodelist.nodes[0].statistics.tx.bytes, 7400661063510)
        self.assertEqual(self.nodelist.nodes[0].statistics.tx.packets, 6481364061)
        self.assertEqual(self.nodelist.nodes[0].statistics.tx.dropped, 8402057)

        self.assertEqual(self.nodelist.nodes[0].statistics.rx.bytes, 811005896638)
        self.assertEqual(self.nodelist.nodes[0].statistics.rx.packets, 4766761900)

        self.assertEqual(self.nodelist.nodes[0].statistics.forward.bytes, 4729587096438)
        self.assertEqual(self.nodelist.nodes[0].statistics.forward.packets, 9701545372)

        self.assertEqual(self.nodelist.nodes[0].statistics.mgmt_tx.bytes, 51533836800)
        self.assertEqual(self.nodelist.nodes[0].statistics.mgmt_tx.packets, 145173118)

        self.assertEqual(self.nodelist.nodes[0].statistics.mgmt_rx.bytes, 1429253469038)
        self.assertEqual(self.nodelist.nodes[0].statistics.mgmt_rx.packets, 3289390848)

    def test_nodeinfo(self):
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.node_id, "daff61000302")

        self.assertEqual(self.nodelist.nodes[0].nodeinfo.network.mac, "da:ff:61:00:03:02")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.network.addresses, [
            ipaddress.ip_address("fe80::543a:9dff:fe11:4238"),
            ipaddress.ip_address("2001:67c:2ed8:6100::3"),
            ipaddress.ip_address("fd01:67c:2ed8:6100::3"),
            ipaddress.ip_address("fe80::d8ff:61ff:fe00:304")
        ])
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.network.mesh["ffda-bat"].tunnel, [
            "da:ff:61:00:03:05",
            "da:ff:61:00:03:03",
            "da:ff:61:01:03:03"
        ])
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.hostname, "gw03.darmstadt.freifunk.net")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.autoupdater.enabled, False)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.autoupdater.branch, None)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.batman_adv.version, "2017.3")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.batman_adv.compat, None)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.fastd.enabled, True)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.fastd.version, "v18")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.firmware.base, "Debian")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.firmware.release, "8.9")
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.software.status_page.api, 0)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.hardware.nproc, 3)
        self.assertEqual(self.nodelist.nodes[0].nodeinfo.vpn, True)



from enum import Enum

from pymeshviewer import Protocol


class LinkType(Enum):
    OTHER = 0
    WIFI = 1
    VPN = 2


class Node:
    def __init__(self, id=None, node_id=None):
        self.id = id
        self.node_id = node_id


class Link:
    def __init__(self, source: str = None, target: str = None, link_type: LinkType = LinkType.OTHER, vpn: bool = False,
                 tq: int = None, source_tq: int = None, target_tq: int = None, bidirect: bool = False):
        self.source = source
        self.target = target
        self.link_type = link_type
        if vpn is True:
            self.link_type = LinkType.OTHER
        self.source_tq = source_tq
        self.target_tq = target_tq
        if tq and source_tq is None and target_tq is None:
            self.target_tq = tq
            self.source_tq = tq
            self.bidirect = False
        self.bidirect = bidirect

        if self.target_tq == 0:
            self.bidirect = False

    @property
    def tq(self):
        return (self.source_tq + self.target_tq) / 2

    @property
    def vpn(self):
        return self.link_type == LinkType.VPN


class ProtocolBatmanAdv:
    def __init__(self, directed: bool = False, graph=None, nodes: list = None, links: list = None):
        if links is None:
            links = []
        if nodes is None:
            nodes = []
        self.directed = directed
        self.graph = graph
        self.nodes = nodes
        self.links = links


class NodeGraph:
    def __init__(self, version: int = 0, protocol: ProtocolBatmanAdv = ProtocolBatmanAdv(),
                 protocol_type: Protocol = None):
        self.version = version
        self.protocol = protocol
        self.protocol_type = protocol_type



import unittest

from pymeshviewer.parser import parse_datetime


class CommonParserTest(unittest.TestCase):
    def test_datetime_parser(self):
        """
        Verify correct datetime parsing
        """
        teststring_1 = "2017-02-10T18:28:24+0000"
        parsed_string_1 = parse_datetime(teststring_1)
        self.assertEqual(parsed_string_1.day, 10)
        self.assertEqual(parsed_string_1.month, 2)
        self.assertEqual(parsed_string_1.year, 2017)
        self.assertEqual(parsed_string_1.hour, 18)
        self.assertEqual(parsed_string_1.minute, 28)
        self.assertEqual(parsed_string_1.second, 24)

        teststring_2 = "2017-10-21T00:33:52+0200"
        parsed_string_2 = parse_datetime(teststring_2)
        self.assertEqual(parsed_string_2.day, 21)
        self.assertEqual(parsed_string_2.month, 10)
        self.assertEqual(parsed_string_2.year, 2017)
        self.assertEqual(parsed_string_2.hour, 00)
        self.assertEqual(parsed_string_2.minute, 33)
        self.assertEqual(parsed_string_2.second, 52)



import unittest

import requests
from pymeshviewer.parser import parse_graph_json


class NodeGraphParserRemoteTest(unittest.TestCase):
    def test_darmstadt(self):
        json = requests.get("https://meshviewer.darmstadt.freifunk.net/data/ffda/graph.json").text
        self.graph_json = parse_graph_json(json)



import unittest

import requests
from pymeshviewer.parser import parse_nodes_json


class NodesJsonParserRemoteTest(unittest.TestCase):
    def test_darmstadt(self):
        json = requests.get("https://meshviewer.darmstadt.freifunk.net/data/ffda/nodes.json").text
        self.node_json = parse_nodes_json(json)



from datetime import datetime
from typing import List

from pymeshviewer.graph import Link
from pymeshviewer.node import Node
from pymeshviewer.nodecollection import NodeCollection


class MeshviewerJSON(NodeCollection):
    def __init__(self, timestamp: datetime, nodes: List[Node], links: List[Link]):
        """
        Constructor for a MeshviewerJSON object
        :param timestamp: timestamp of MeshviewerJSON
        :param nodes: list of nodes
        :param links: list of links
        """
        super().__init__(nodes)
        self.timestamp = timestamp
        self.nodes = nodes
        self.links = links



from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

setup(
    name='pymeshviewer',
    version='0.1.1',

    description='Parser for Meshviewer nodelists and topology files',

    # The project's main homepage.
    url='https://github.com/blocktrron/pymeshviewer',

    # Author details
    author='blocktrron',
    author_email='david@darmstadt.freifunk.net',

    # Choose your license
    license='AGPLv3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='development meshviewer freifunk',
    packages=['pymeshviewer'],
    install_requires=['requests'],
)



from typing import Dict, Optional, List, Tuple

from pymeshviewer.node import Node
from pymeshviewer.util import calculate_distance


class NodeCollection:
    def __init__(self, nodes: List[Node]):
        self.nodes = nodes

    @property
    def node_count(self) -> int:
        """
        Total number of nodes in nodelist
        :return: total number of nodes in nodelist
        """
        return len(self.nodes)

    @property
    def online(self) -> List[Node]:
        return [n for n in self.nodes if n.online]

    @property
    def offline(self) -> List[Node]:
        return [n for n in self.nodes if not n.online]

    @property
    def models(self) -> Dict[str, Node]:
        """
        Nodes by their models
        :return: nodes ordered by model
        """
        models = {}
        for node in self.nodes:
            if node.nodeinfo.hardware.model not in models:
                models[node.nodeinfo.hardware.model] = []
            models[node.nodeinfo.hardware.model].append(node)
        return models

    @property
    def vpn_enabled(self):
        """
        Nodes with enabled vpn
        :return: nodes with enabled vpn
        """
        nodes = []
        for node in self.nodes:
            if node.vpn_enabled is True:
                nodes.append(node)
        return nodes

    @property
    def established_vpn_connection(self) -> List[Node]:
        """
        Nodes with established vpn connection
        :return: nodes with established vpn connection
        """
        nodes = []
        for node in self.nodes:
            if node.vpn_active:
                nodes.append(node)
        return nodes

    @property
    def model_stats(self) -> Dict[str, int]:
        """
        Models and their quantity
        :return: models and their quantity
        """
        models = {}
        for node in self.nodes:
            model = node.nodeinfo.hardware.model
            if model is None:
                continue
            if model not in models:
                models[model] = 0
            models[model] += 1
        return models

    def get_closest_node(self, latitude, longitude, online=None) -> Tuple[Node, float]:
        """
        Returns the closest node and the distance relative to provided coordinates
        :param latitude: latitude
        :param longitude: longitude
        :param online: indicates if the closest online/offline node is looked for, ignores status if unset
        :return: tuple consisting of closest node and distance in km
        """
        if online is None:
            nodes = self.nodes
        elif online is True:
            nodes = self.online
        elif online is False:
            node = self.offline

        closest = None
        closest_distance = None
        for node in self.nodes:
            if node.nodeinfo.location is None:
                continue
            distance = calculate_distance(latitude, longitude, node.nodeinfo.location.latitude,
                                          node.nodeinfo.location.longitude)
            if closest_distance is None or closest_distance > distance:
                closest_distance = distance
                closest = node

        return closest, closest_distance

    def get_node(self, node_id: str) -> Optional[Node]:
        """
        Returns node identified by its node_id
        :param node_id: node_id of desired node
        :return: Node if node present in nodelist, None otherwise
        """
        for node in self.nodes:
            if node.nodeinfo.node_id == node_id:
                return node
        return None

    def get_hostname(self, hostname: str) -> List[Node]:
        """
        Returns all nodes which hostname contain a specified string as a List
        :param hostname: hostname of desired node(s)
        :return: list of nodes with specified hostname
        """
        nodes = [n for n in self.nodes if hostname in n.nodeinfo.hostname]
        sorted_x = sorted(nodes,
                          key=lambda n: len(n.nodeinfo.hostname) - len(hostname) * n.nodeinfo.hostname.count(hostname))
        return sorted_x



import unittest

import requests
from pymeshviewer.parser import parse_meshviewer_json


class MeshviewerJsonParserRemoteTest(unittest.TestCase):
    def test_darmstadt(self):
        json = requests.get("https://meshviewer.darmstadt.freifunk.net/data/ffda/meshviewer.json").text
        self.meshviewer_json = parse_meshviewer_json(json)

    def test_bremen(self):
        json = requests.get("https://downloads.bremen.freifunk.net/data/meshviewer.json").text
        self.meshviewer_json = parse_meshviewer_json(json)



import os
import unittest

from pymeshviewer.parser import parse_meshviewer_json


class NodecollectionTest(unittest.TestCase):
    def setUp(self):
        """
        Initialize Tests
        """
        nodelist_file = open(
            os.path.dirname(os.path.realpath(__file__)) + os.sep + "testfiles/yanic_example_town_meshviewer.json", "r")
        self.meshviewer_json = parse_meshviewer_json(nodelist_file.read())
        nodelist_file.close()

    def test_closest_node(self):
        """ Assert that closest node is returned """
        node, distance = self.meshviewer_json.get_closest_node(49.835076144, 8.697116375)
        self.assertEqual(node, self.meshviewer_json.get_node("c04a00dd692a"))
        self.assertAlmostEqual(distance, 0.304, 3)

    def test_online_nodes(self):
        """ Asserts that all offline nodes are returned """
        self.assertEqual(len(self.meshviewer_json.online), 4)

    def test_offline_nodes(self):
        self.assertEqual(len(self.meshviewer_json.offline), 0)

    def test_get_hostname(self):
        self.assertEqual(len(self.meshviewer_json.get_hostname("invalid")), 0)
        self.assertEqual(self.meshviewer_json.get_hostname("e"),
                         [self.meshviewer_json.get_node("60e3272f92b2"),
                          self.meshviewer_json.get_node("c04a00dd692a"),
                          self.meshviewer_json.get_node("daff61000302"),
                          self.meshviewer_json.get_node("daff61000402")])  # Check sorting
        self.assertEqual(self.meshviewer_json.get_hostname("tagungshotel"),
                         [self.meshviewer_json.get_node("60e3272f92b2")])
        self.assertEqual(self.meshviewer_json.get_hostname("64367-tagungshotel02"),
                         [self.meshviewer_json.get_node("60e3272f92b2")])
        self.assertEqual(self.meshviewer_json.get_hostname("64367-tagungshotel02 "), [])
        self.assertEqual(self.meshviewer_json.get_hostname("block"), [self.meshviewer_json.get_node("c04a00dd692a")])
